@@ -1,12 +1,13 @@
 package Unix::Passwd::File;
 
+our $DATE = '2015-03-06'; # DATE
+our $VERSION = '0.13'; # VERSION
+
 use 5.010001;
 use strict;
 use warnings;
 use experimental 'smartmatch';
-use Log::Any '$log';
-
-our $VERSION = '0.12'; # VERSION
+use Log::Any::IfLOG '$log';
 
 use File::Flock;
 use List::Util qw(max first);
@@ -40,10 +41,16 @@ our @EXPORT_OK = qw(
 
 our %SPEC;
 
+$SPEC{':package'} = {
+    v => 1.1,
+    summary => 'Manipulate /etc/{passwd,shadow,group,gshadow} entries',
+};
+
 my %common_args = (
     etc_dir => {
         summary => 'Specify location of passwd files',
         schema  => ['str*' => {default=>'/etc'}],
+        tags    => ['common'],
     },
 );
 my %write_args = (
@@ -238,9 +245,10 @@ sub _backup {
     [200];
 }
 
-# all public functions in this module uses the _routine, which contains the
-# basic flow, to avoid duplication of code. _routine accept these special
-# arguments for flow control:
+# all public functions in this module use the _routine(), which contains the
+# basic flow, to avoid duplication of code. admittedly this makes _routine()
+# quite convoluted, as it tries to accomodate all the functions' logic in a
+# single routine. _routine() accepts these special arguments for flow control:
 #
 # - _read_shadow   = 0*/1/2 (2 means optional, don't exit if fail)
 # - _read_passwd   = 0*/1
@@ -256,13 +264,13 @@ sub _backup {
 # - _write_group   = 0*/1
 #
 # all the hooks are fed $stash, sort of like a bag or object containing all
-# data. should return enveloped response. _routine will return with response if
-# response is non success. _routine will also return immediately if $stash{exit}
-# is set.
+# data. should return enveloped response. _routine() will return with response
+# if response is non success. _routine() will also return immediately if
+# $stash{exit} is set.
 #
-# to write, we open once but with mode +< instead of <. we read first then we
-# seek back to beginning and write from in-memory data. if
-# $stash{write_passwd} and so on is set to false, routine cancel the write
+# to write, we open once but with mode '+<' instead of '<'. we read first then
+# we seek back to beginning and write from in-memory data. if
+# $stash{write_passwd} and so on is set to false, _routine() cancels the write
 # (can be used e.g. when there is no change so no need to write).
 #
 # final result is in $stash{res} or non-success result returned by hook.
@@ -498,9 +506,9 @@ $SPEC{list_users} = {
             description => <<'_',
 
 By default, when `detail=>1`, a hashref is returned for each entry containing
-field names and its values, e.g. `{user=>"neil", pass=>"x", uid=>500, ...}`.
-With `with_field_names=>0`, an arrayref is returned instead: `["neil", "x", 500,
-...]`.
+field names and its values, e.g. `{user=>"titin", pass=>"x", uid=>500, ...}`.
+With `with_field_names=>0`, an arrayref is returned instead: `["titin", "x",
+500, ...]`.
 
 _
         },
@@ -565,8 +573,8 @@ _
             description => <<'_',
 
 By default, a hashref is returned containing field names and its values, e.g.
-`{user=>"neil", pass=>"x", uid=>500, ...}`. With `with_field_names=>0`, an
-arrayref is returned instead: `["neil", "x", 500, ...]`.
+`{user=>"titin", pass=>"x", uid=>500, ...}`. With `with_field_names=>0`, an
+arrayref is returned instead: `["titin", "x", 500, ...]`.
 
 _
         },
@@ -647,9 +655,9 @@ $SPEC{list_groups} = {
             description => <<'_',
 
 By default, when `detail=>1`, a hashref is returned for each entry containing
-field names and its values, e.g. `{group=>"neil", pass=>"x", gid=>500, ...}`.
-With `with_field_names=>0`, an arrayref is returned instead: `["neil", "x", 500,
-...]`.
+field names and its values, e.g. `{group=>"titin", pass=>"x", gid=>500, ...}`.
+With `with_field_names=>0`, an arrayref is returned instead: `["titin", "x",
+500, ...]`.
 
 _
         },
@@ -714,8 +722,8 @@ _
             description => <<'_',
 
 By default, a hashref is returned containing field names and its values, e.g.
-`{group=>"neil", pass=>"x", gid=>500, ...}`. With `with_field_names=>0`, an
-arrayref is returned instead: `["neil", "x", 500, ...]`.
+`{group=>"titin", pass=>"x", gid=>500, ...}`. With `with_field_names=>0`, an
+arrayref is returned instead: `["titin", "x", 500, ...]`.
 
 _
         },
@@ -870,9 +878,9 @@ $SPEC{get_user_groups} = {
             description => <<'_',
 
 By default, when `detail=>1`, a hashref is returned for each entry containing
-field names and its values, e.g. `{group=>"neil", pass=>"x", gid=>500, ...}`.
-With `with_field_names=>0`, an arrayref is returned instead: `["neil", "x", 500,
-...]`.
+field names and its values, e.g. `{group=>"titin", pass=>"x", gid=>500, ...}`.
+With `with_field_names=>0`, an arrayref is returned instead: `["titin", "x",
+500, ...]`.
 
 _
         },
@@ -1827,7 +1835,7 @@ Unix::Passwd::File - Manipulate /etc/{passwd,shadow,group,gshadow} entries
 
 =head1 VERSION
 
-This document describes version 0.12 of Unix::Passwd::File (from Perl distribution Unix-Passwd-File), released on 2014-05-17.
+This document describes version 0.13 of Unix::Passwd::File (from Perl distribution Unix-Passwd-File), released on 2015-03-06.
 
 =head1 SYNOPSIS
 
@@ -1845,38 +1853,38 @@ This document describes version 0.12 of Unix::Passwd::File (from Perl distributi
      # [200, "OK", [["root", "x", 0, ...], ...]]
 
  # get user/group information
- $res = get_group(user=>"buzz"); # [200, "OK", {user=>"buzz", uid=>501, ...}]
- $res = get_user(user=>"neil");  # [404, "Not found"]
+ $res = get_group(user=>"paijo"); # [200, "OK", {user=>"paijo", uid=>501, ...}]
+ $res = get_user(user=>"titin");  # [404, "Not found"]
 
  # check whether user/group exists
- say user_exists(user=>"buzz");   # 1
- say group_exists(group=>"neil"); # 0
+ say user_exists(user=>"paijo");   # 1
+ say group_exists(group=>"titin"); # 0
 
  # get all groups that user is member of
- $res = get_user_groups(user=>"buzz"); # [200, "OK", ["buzz", "nasa"]]
+ $res = get_user_groups(user=>"paijo"); # [200, "OK", ["paijo", "satpam"]]
 
  # check whether user is member of a group
- $res = is_member(user=>"buzz", group=>"nasa"); # 1
+ $res = is_member(user=>"paijo", group=>"satpam"); # 1
 
  # adding user/group, by default adding user will also add a group with the same
  # name
- $res = add_user (user =>"steven", ...); # [200, "OK", {uid=>540, gid=>541}]
- $res = add_group(group=>"steven", ...); # [412, "Group already exists"]
+ $res = add_user (user =>"ujang", ...); # [200, "OK", {uid=>540, gid=>541}]
+ $res = add_group(group=>"ujang", ...); # [412, "Group already exists"]
 
  # modify user/group
- $res = modify_user(user=>"steven", home=>"/newhome/steven"); # [200, "OK"]
- $res = modify_group(group=>"neil"); # [404, "Not found"]
+ $res = modify_user(user=>"ujang", home=>"/newhome/ujang"); # [200, "OK"]
+ $res = modify_group(group=>"titin"); # [404, "Not found"]
 
  # deleting user will also delete user's group
- $res = delete_user(user=>"neil");
+ $res = delete_user(user=>"titin");
 
  # change user password
- $res = set_user_password(user=>"steven", pass=>"foobar");
- $res = modify_user(user=>"steven", pass=>"foobar"); # same thing
+ $res = set_user_password(user=>"ujang", pass=>"foobar");
+ $res = modify_user(user=>"ujang", pass=>"foobar"); # same thing
 
  # add/delete user to/from group
- $res = add_user_to_group(user=>"steven", group=>"wheel");
- $res = delete_user_from_group(user=>"steven", group=>"wheel");
+ $res = add_user_to_group(user=>"ujang", group=>"wheel");
+ $res = delete_user_from_group(user=>"ujang", group=>"wheel");
 
  # others
  $res = get_max_uid(); # [200, "OK", 65535]
@@ -1907,14 +1915,14 @@ Add or delete user from one or several groups.
 This can be used to reduce several C<add_user_to_group()> and/or
 C<delete_user_from_group()> calls to a single call. So:
 
-    add_delete_user_groups(user=>'u',add_to=>['a','b'],delete_from=>['c','d']);
+ add_delete_user_groups(user=>'u',add_to=>['a','b'],delete_from=>['c','d']);
 
 is equivalent to:
 
-    add_user_to_group     (user=>'u', group=>'a');
-    add_user_to_group     (user=>'u', group=>'b');
-    delete_user_from_group(user=>'u', group=>'c');
-    delete_user_from_group(user=>'u', group=>'d');
+ add_user_to_group     (user=>'u', group=>'a');
+ add_user_to_group     (user=>'u', group=>'b');
+ delete_user_from_group(user=>'u', group=>'c');
+ delete_user_from_group(user=>'u', group=>'d');
 
 except that C<add_delete_user_groups()> does it in one pass.
 
@@ -1922,11 +1930,11 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<add_to> => I<array> (default: [])
+=item * B<add_to> => I<array[str]> (default: [])
 
 List of group names to add the user as member of.
 
-=item * B<delete_from> => I<array> (default: [])
+=item * B<delete_from> => I<array[str]> (default: [])
 
 List of group names to remove the user as member of.
 
@@ -1938,8 +1946,6 @@ Specify location of passwd files.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -1949,6 +1955,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 add_group(%args) -> [status, msg, result, meta]
 
@@ -1997,8 +2005,6 @@ returned.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2008,6 +2014,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 add_user(%args) -> [status, msg, result, meta]
 
@@ -2119,8 +2127,6 @@ Adding a new user with duplicate UID is allowed.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2130,6 +2136,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 add_user_to_group(%args) -> [status, msg, result, meta]
 
@@ -2149,8 +2157,6 @@ Specify location of passwd files.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2160,6 +2166,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 delete_group(%args) -> [status, msg, result, meta]
 
@@ -2184,8 +2192,6 @@ Specify location of passwd files.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2195,6 +2201,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 delete_user(%args) -> [status, msg, result, meta]
 
@@ -2219,8 +2227,6 @@ Specify location of passwd files.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2230,6 +2236,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 delete_user_from_group(%args) -> [status, msg, result, meta]
 
@@ -2249,8 +2257,6 @@ Specify location of passwd files.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2260,6 +2266,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 get_group(%args) -> [status, msg, result, meta]
 
@@ -2286,13 +2294,11 @@ Specify location of passwd files.
 If false, don't return hash.
 
 By default, a hashref is returned containing field names and its values, e.g.
-C<{group=>"neil", pass=>"x", gid=>500, ...}>. With C<with_field_names=>0>, an
-arrayref is returned instead: C<["neil", "x", 500, ...]>.
+C<< {group=E<gt>"titin", pass=E<gt>"x", gid=E<gt>500, ...} >>. With C<< with_field_names=E<gt>0 >>, an
+arrayref is returned instead: C<["titin", "x", 500, ...]>.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2302,6 +2308,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 get_max_gid(%args) -> [status, msg, result, meta]
 
@@ -2317,8 +2325,6 @@ Specify location of passwd files.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2328,6 +2334,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 get_max_uid(%args) -> [status, msg, result, meta]
 
@@ -2343,8 +2351,6 @@ Specify location of passwd files.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2354,6 +2360,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 get_user(%args) -> [status, msg, result, meta]
 
@@ -2380,13 +2388,11 @@ Specify location of passwd files.
 If false, don't return hash.
 
 By default, a hashref is returned containing field names and its values, e.g.
-C<{user=>"neil", pass=>"x", uid=>500, ...}>. With C<with_field_names=>0>, an
-arrayref is returned instead: C<["neil", "x", 500, ...]>.
+C<< {user=E<gt>"titin", pass=E<gt>"x", uid=E<gt>500, ...} >>. With C<< with_field_names=E<gt>0 >>, an
+arrayref is returned instead: C<["titin", "x", 500, ...]>.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2396,6 +2402,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 get_user_groups(%args) -> [status, msg, result, meta]
 
@@ -2419,15 +2427,13 @@ Specify location of passwd files.
 
 If false, don't return hash for each entry.
 
-By default, when C<detail=>1>, a hashref is returned for each entry containing
-field names and its values, e.g. C<{group=>"neil", pass=>"x", gid=>500, ...}>.
-With C<with_field_names=>0>, an arrayref is returned instead: C<["neil", "x", 500,
-...]>.
+By default, when C<< detail=E<gt>1 >>, a hashref is returned for each entry containing
+field names and its values, e.g. C<< {group=E<gt>"titin", pass=E<gt>"x", gid=E<gt>500, ...} >>.
+With C<< with_field_names=E<gt>0 >>, an arrayref is returned instead: C<["titin", "x",
+500, ...]>.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2437,6 +2443,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 group_exists(%args) -> bool
 
@@ -2456,7 +2464,7 @@ Specify location of passwd files.
 
 =back
 
-Return value:
+Return value:  (bool)
 
 
 =head2 is_member(%args) -> bool
@@ -2477,7 +2485,7 @@ Specify location of passwd files.
 
 =back
 
-Return value:
+Return value:  (bool)
 
 
 =head2 list_groups(%args) -> [status, msg, result, meta]
@@ -2500,15 +2508,13 @@ Specify location of passwd files.
 
 If false, don't return hash for each entry.
 
-By default, when C<detail=>1>, a hashref is returned for each entry containing
-field names and its values, e.g. C<{group=>"neil", pass=>"x", gid=>500, ...}>.
-With C<with_field_names=>0>, an arrayref is returned instead: C<["neil", "x", 500,
-...]>.
+By default, when C<< detail=E<gt>1 >>, a hashref is returned for each entry containing
+field names and its values, e.g. C<< {group=E<gt>"titin", pass=E<gt>"x", gid=E<gt>500, ...} >>.
+With C<< with_field_names=E<gt>0 >>, an arrayref is returned instead: C<["titin", "x",
+500, ...]>.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2518,6 +2524,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 list_users(%args) -> [status, msg, result, meta]
 
@@ -2539,15 +2547,13 @@ Specify location of passwd files.
 
 If false, don't return hash for each entry.
 
-By default, when C<detail=>1>, a hashref is returned for each entry containing
-field names and its values, e.g. C<{user=>"neil", pass=>"x", uid=>500, ...}>.
-With C<with_field_names=>0>, an arrayref is returned instead: C<["neil", "x", 500,
-...]>.
+By default, when C<< detail=E<gt>1 >>, a hashref is returned for each entry containing
+field names and its values, e.g. C<< {user=E<gt>"titin", pass=E<gt>"x", uid=E<gt>500, ...} >>.
+With C<< with_field_names=E<gt>0 >>, an arrayref is returned instead: C<["titin", "x",
+500, ...]>.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2557,6 +2563,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 list_users_and_groups(%args) -> [status, msg, result, meta]
 
@@ -2584,8 +2592,6 @@ If false, don't return hash for each entry.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2595,6 +2601,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 modify_group(%args) -> [status, msg, result, meta]
 
@@ -2644,8 +2652,6 @@ Password, generally should be "x" which means password is encrypted in gshadow.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2655,6 +2661,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 modify_user(%args) -> [status, msg, result, meta]
 
@@ -2736,8 +2744,6 @@ User (login) name.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2747,6 +2753,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 set_user_groups(%args) -> [status, msg, result, meta]
 
@@ -2760,7 +2768,7 @@ Arguments ('*' denotes required arguments):
 
 Specify location of passwd files.
 
-=item * B<groups>* => I<array> (default: [])
+=item * B<groups>* => I<array[str]> (default: [])
 
 List of group names that user is member of.
 
@@ -2770,8 +2778,6 @@ Aside from this list, user will not belong to any other group.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2781,6 +2787,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 set_user_password(%args) -> [status, msg, result, meta]
 
@@ -2807,8 +2815,6 @@ Specify location of passwd files.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -2818,6 +2824,8 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value:  (any)
+
 
 =head2 user_exists(%args) -> bool
 
@@ -2837,7 +2845,7 @@ Specify location of passwd files.
 
 =back
 
-Return value:
+Return value:  (bool)
 
 =head1 SEE ALSO
 
@@ -2888,7 +2896,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Unix-Passw
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/sharyanto/perl-Unix-Passwd-File>.
+Source repository is at L<https://github.com/perlancar/perl-Unix-Passwd-File>.
 
 =head1 BUGS
 
@@ -2900,11 +2908,11 @@ feature.
 
 =head1 AUTHOR
 
-Steven Haryanto <stevenharyanto@gmail.com>
+perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Steven Haryanto.
+This software is copyright (c) 2015 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
